<?php

namespace GGM\ProductManagerImplementation\ProductManagerMagento2;

use Magento;
use Magento\Backend\Model\UrlInterface;
use Magento\Catalog\Model\Product\Media\Config;
use Magento\Catalog\Model\ResourceModel\Category\Attribute\CollectionFactory;
use Magento\Catalog\Model\ResourceModel\Product\Attribute\CollectionFactory as AttributeCollectionFactory;
use Magento\CatalogInventory\Api\StockConfigurationInterface;
use Magento\Customer\Model\ResourceModel\Group\Collection;
use Magento\Framework\App\ProductMetadataInterface;
use Magento\Framework\App\ResourceConnection;
use Magento\GiftMessage\Model\GiftMessageConfigProvider;
use Magento\Store\Model\StoreManager;
use stdClass;

/**
 * Class ProductManagerConfigHelper
 */
class ProductManagerConfigHelper
{
    const disable_grid_editing = false;                     //default value: false. If true, grid editing is disabled in Product Manager for Magento
    const disable_grid_mass_action = false;                 //default value: false. If true, copy and paste to multiple products in Product Manager for Magento
    const disable_grid_configuration = false;               //default value: false. If true, "Manage tabs and columns" menu is disabled in Product Manager for Magento
    const force_simplified_view_in_editors = false;         //default value: false. If true, simplified view mode is forced when editing product in Product Manager for Magento
    protected $_storeManager;
    protected $_productMediaConfig;
    protected $_productMetadata;
    protected $_giftMessageConfigProvider;
    protected $_resourceConnection;
    protected $_stockConfiguration;
    protected $_backendUrl;
    protected $_productAttributeCollectionFactory;
    protected $_categoryAttributeCollectionFactory;
    protected $_customerGroupsCollection;
    protected $_indexerCollectionFactory;

    public function __construct(
        StoreManager $storeManager,
        Config $productMediaConfig,
        ProductMetadataInterface $productMetadataInterface,
        GiftMessageConfigProvider $giftMessageConfigProvider,
        ResourceConnection $resourceConnection,
        StockConfigurationInterface $stockConfiguration,
        UrlInterface $backendUrl,
        AttributeCollectionFactory $productAttributeCollectionFactory,
        CollectionFactory $categoryAttributeCollectionFactory,
        Collection $customerGroupsCollection,
        Magento\Indexer\Model\Indexer\CollectionFactory $indexerCollectionFactory
    ) {
        $this->_storeManager = $storeManager;
        $this->_productMediaConfig = $productMediaConfig;
        $this->_productMetadata = $productMetadataInterface;
        $this->_giftMessageConfigProvider = $giftMessageConfigProvider;
        $this->_resourceConnection = $resourceConnection;
        $this->_stockConfiguration = $stockConfiguration;
        $this->_backendUrl = $backendUrl;
        $this->_productAttributeCollectionFactory = $productAttributeCollectionFactory;
        $this->_categoryAttributeCollectionFactory = $categoryAttributeCollectionFactory;
        $this->_customerGroupsCollection = $customerGroupsCollection;
        $this->_indexerCollectionFactory = $indexerCollectionFactory;
    }

    public function getConfig(&$jsonRpcResult)
    {
        $jsonRpcResult->result = new stdClass();

        $jsonRpcResult->result->magento_version = $this->_productMetadata->getVersion();
        $jsonRpcResult->result->php_version = phpversion();
        $jsonRpcResult->result->max_execution_time = ini_get('max_execution_time');
        $jsonRpcResult->result->max_input_time = ini_get('max_input_time');
        $jsonRpcResult->result->memory_limit = ini_get('memory_limit');
        $jsonRpcResult->result->post_max_size = ini_get('post_max_size');
        $jsonRpcResult->result->upload_max_filesize = ini_get('upload_max_filesize');
        $jsonRpcResult->result->zlib_output_compression = ini_get('zlib.output_compression');

        $tableName = $this->_resourceConnection->getTableName('core_config_data');
        $pos = strrpos($tableName, 'core_config_data');
        if ($pos === false) {
            $prefix = '';
        } else {
            $prefix = substr($tableName, 0, $pos);
        }

        $jsonRpcResult->result->table_prefix = $prefix;

        $jsonRpcResult->result->media_product_base_url = $this->_productMediaConfig->getBaseMediaUrl();
        $jsonRpcResult->result->media_product_base_path = $this->_productMediaConfig->getBaseMediaPath();

        $baseMediaURL = $this->_storeManager->getStore()->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_MEDIA);
        $baseMediaDir = $this->_storeManager->getStore()->getBaseMediaDir();

        $jsonRpcResult->result->media_category_base_url = $baseMediaURL . 'catalog/category';
        $jsonRpcResult->result->media_category_base_path = $baseMediaDir . '/' . 'catalog' . '/' . 'category';

        $jsonRpcResult->result->installation_in_pub_folder = !file_exists('app/bootstrap.php');

        if ($jsonRpcResult->result->installation_in_pub_folder) {
            $jsonRpcResult->result->original_media_category_base_url = $jsonRpcResult->result->media_category_base_url;
            $jsonRpcResult->result->original_media_product_base_url = $jsonRpcResult->result->media_product_base_url;

            if (file_exists('media/catalog')) {
                $jsonRpcResult->result->media_category_base_url = str_replace('/pub/media/catalog/', '/media/catalog/', $jsonRpcResult->result->media_category_base_url);
                $jsonRpcResult->result->media_product_base_url = str_replace('/pub/media/catalog/', '/media/catalog/', $jsonRpcResult->result->media_product_base_url);
            }
        }

        $jsonRpcResult->result->locale_code = $this->_storeManager->getStore()->getLocaleCode();
        $baseCurrency = $this->_storeManager->getStore()->getBaseCurrency();

        $jsonRpcResult->result->base_currency = $this->_storeManager->getStore()->getBaseCurrencyCode();
        $storeCollection = $this->_storeManager->getStores(true);

        $jsonRpcResult->result->stores = array_keys($storeCollection);
        $jsonRpcResult->result->storeCollection = $storeCollection;

        $jsonRpcResult->result->cataloginventory_item_options_manage_stock = $this->_stockConfiguration->getManageStock();
        $jsonRpcResult->result->cataloginventory_item_options_backorders = $this->_stockConfiguration->getBackorders();
        $jsonRpcResult->result->cataloginventory_item_options_max_sale_qty = $this->_stockConfiguration->getMaxSaleQty();
        $jsonRpcResult->result->cataloginventory_item_options_min_qty = $this->_stockConfiguration->getMinQty();
        $jsonRpcResult->result->cataloginventory_item_options_min_sale_qty = $this->_stockConfiguration->getMinSaleQty();
        $jsonRpcResult->result->cataloginventory_item_options_notify_stock_qty = $this->_stockConfiguration->getNotifyStockQty();
        $jsonRpcResult->result->cataloginventory_item_options_enable_qty_increments = $this->_stockConfiguration->getEnableQtyIncrements();
        $jsonRpcResult->result->cataloginventory_item_options_qty_increments = $this->_stockConfiguration->getQtyIncrements();

        $indexerCollection = $this->_indexerCollectionFactory->create();
        $jsonRpcResult->result->indexers = array();

        foreach ($indexerCollection->getItems() as $indexer) {
            $indexerInfo = new stdClass();
            $indexerInfo->indexer_id = $indexer->getId();
            $indexerInfo->title = $indexer->getTitle();
            $indexerInfo->description = $indexer->getDescription();

            $jsonRpcResult->result->indexers[] = $indexerInfo;
        }

        $jsonRpcResult->result->product_manager_configuration = new stdClass();
        $jsonRpcResult->result->product_manager_configuration->permissions = new stdClass();
        $jsonRpcResult->result->product_manager_configuration->permissions->disable_grid_editing = self::disable_grid_editing;
        $jsonRpcResult->result->product_manager_configuration->permissions->disable_grid_mass_action = self::disable_grid_mass_action;
        $jsonRpcResult->result->product_manager_configuration->permissions->disable_grid_configuration = self::disable_grid_configuration;
        $jsonRpcResult->result->product_manager_configuration->permissions->force_simplified_view_in_editors = self::force_simplified_view_in_editors;
    }

    public function getAttributeSourceModels($entityType, $attributeCollectionFactory)
    {
        $models = array();

        $attributeCollection = $attributeCollectionFactory->create();
        $attributeCollection->addFieldToFilter('source_model', array('neq' => 'NULL'));
        $attributes = $attributeCollection->getItems();

        foreach ($attributes as $attribute) {
            $sourceModel = new stdClass();

            try {
                $sourceModel->entity_type = $entityType;
                $sourceModel->attribute_id = $attribute->getAttributeId();
                $sourceModel->attribute_code = $attribute->getAttributeCode();
                $sourceModel->attribute_frontend_label = $attribute->getFrontendLabel();
                $sourceModel->model_class = $attribute->getSourceModel();
                $sourceModel->options = $attribute->getSource()->getAllOptions();
            } catch (\Exception $e) {
                $sourceModel->error = 'Exception: ' . $e->getMessage() . ' Trace: ' . $e->getTraceAsString();
            }
            $models[] = $sourceModel;
        }

        return $models;
    }

    public function modelGetAllOptions($modelClass)
    {
        $sourceModel = new stdClass();
        $sourceModel->model_class = $modelClass;

        try {
            $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
            $model = $objectManager->create($modelClass);

            if ($model) {
                $sourceModel->options = $model->getAllOptions(true);
            }
        } catch (\Exception $e) {
            $sourceModel->error = 'Exception: ' . $e->getMessage() . ' Trace: ' . $e->getTraceAsString();
        }

        return $sourceModel;
    }

    public function modelGetOptions($modelClass)
    {
        $sourceModel = new stdClass();
        $sourceModel->model_class = $modelClass;

        try {
            $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
            $model = $objectManager->create($modelClass);

            if ($model) {
                $sourceModel->options = $model->getOptions();
            }
        } catch (\Exception $e) {
            $sourceModel->error = 'Exception: ' . $e->getMessage() . ' Trace: ' . $e->getTraceAsString();
        }

        return $sourceModel;
    }

    public function getSourceModels(&$jsonRpcResult, $store_id, $locale_code)
    {
        $jsonRpcResult->result = new stdClass();

        $models = array();

        $models[] = $this->modelGetAllOptions('Magento\Bundle\Model\Product\Attribute\Source\Price\View');
        $models[] = $this->modelGetAllOptions('Magento\Bundle\Model\Product\Attribute\Source\Shipment\Type');
        $models[] = $this->modelGetAllOptions('Magento\Catalog\Model\Category\Attribute\Source\Layout');
        $models[] = $this->modelGetAllOptions('Magento\Catalog\Model\Category\Attribute\Source\Mode');
        $models[] = $this->modelGetAllOptions('Magento\Catalog\Model\Category\Attribute\Source\Page');
        $models[] = $this->modelGetAllOptions('Magento\Catalog\Model\Entity\Product\Attribute\Design\Options\Container');
        $models[] = $this->modelGetAllOptions('Magento\Catalog\Model\Product\Attribute\Source\Boolean');
        $models[] = $this->modelGetAllOptions('Magento\Catalog\Model\Product\Attribute\Source\Countryofmanufacture');
        $models[] = $this->modelGetAllOptions('Magento\Catalog\Model\Product\Attribute\Source\Layout');
        $models[] = $this->modelGetAllOptions('Magento\Catalog\Model\Product\Attribute\Source\Status');
        $models[] = $this->modelGetOptions('Magento\Catalog\Model\Product\Type');
        $models[] = $this->modelGetAllOptions('Magento\Catalog\Model\Product\Visibility');
        $models[] = $this->modelGetAllOptions('Magento\CatalogInventory\Model\Source\Stock');
        $models[] = $this->modelGetAllOptions('Magento\Eav\Model\Entity\Attribute\Source\Boolean');
        $models[] = $this->modelGetAllOptions('Magento\Msrp\Model\Product\Attribute\Source\Type\Price');
        $models[] = $this->modelGetAllOptions('Magento\Tax\Model\TaxClass\Source\Product');
        $models[] = $this->modelGetAllOptions('Magento\Theme\Model\Theme\Source\Theme');

        $productAttributesSourceModels = $this->getAttributeSourceModels('product', $this->_productAttributeCollectionFactory);
        $categoryAttributesSourceModels = $this->getAttributeSourceModels('category', $this->_categoryAttributeCollectionFactory);

        $jsonRpcResult->result->source_models = array_merge($models, $productAttributesSourceModels, $categoryAttributesSourceModels);

        $customerGroups = $this->_customerGroupsCollection->toOptionArray();
        $jsonRpcResult->result->customer_groups = $customerGroups;
    }
}