<?php

namespace GGM\ProductManagerImplementation\ProductManagerMagento2;

use Magento\Catalog\Model\Product\Media\Config;
use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\Filesystem;
use Magento\MediaStorage\Model\File\Uploader;
use Magento\MediaStorage\Service\ImageResize;
use Magento\Store\Model\StoreManager;

class ProductManagerImageHelper
{
    protected $_storeManager;
    protected $_productMediaConfig;
    protected $_mediaDirectory;
    protected $_uploader;
    protected $_imageResize = null;

    public function __construct(
        StoreManager $storeManager,
        Config $productMediaConfig,
        Filesystem $filesystem,
        ImageResize $imageResize = null
    ) {
        $this->_storeManager = $storeManager;
        $this->_productMediaConfig = $productMediaConfig;
        $this->_mediaDirectory = $filesystem->getDirectoryWrite(DirectoryList::MEDIA);
        if (!$imageResize && class_exists(ImageResize::class)) {
            $this->_imageResize = ObjectManager::getInstance()->get(ImageResize::class);
        }
    }

    public function uploadImage($jsonRpcResult, $type, $filename, $data, $lastModificationTime, $failIfFileExists, $useDispretionPath)
    {
        $nbBytes = 0;
        $error = null;
        $errorCode = null;
        $noDuplicatedFileName = '';
        $remoteFileSize = -1;
        $remoteLastModificationTime = 0;
        $imagesDeletedInCache = 0;

        $base = '';
        if ($type == 'product') {
            $base = $this->_productMediaConfig->getBaseMediaPath();
        }
        if ($type == 'category') {
            $base = 'catalog/category';
        }

        $fileName = Uploader::getCorrectFileName($filename);

        if ($useDispretionPath) {
            $dispretionPath = Uploader::getDispretionPath($fileName);
            $fileNameWithDispretionPath = $dispretionPath . '/' . $fileName;
            $filePath = $base . $fileNameWithDispretionPath;
        } else {
            $fileNameWithDispretionPath = $fileName;
            $filePath = $base . '/' .  $fileNameWithDispretionPath;
        }


        $absoluteFilePath = $this->_mediaDirectory->getAbsolutePath($filePath);

        if ($this->_mediaDirectory->isExist($filePath)) {
            $remoteLastModificationTime = filemtime($absoluteFilePath);

            if ($failIfFileExists) {
                $error = 'ERROR_FILE_AREADY_EXISTS';

                $noDuplicatedFileName = Uploader::getNewFileName($absoluteFilePath);
            } elseif (!$this->_mediaDirectory->isWritable($filePath)) {
                $error = 'ERROR_FILE_NOT_WRITABLE';
            }
        } else {
            $destinationDirectory = dirname($filePath);

            if ($this->_mediaDirectory->isExist($destinationDirectory) && !$this->_mediaDirectory->isWritable($destinationDirectory)) {
                $error = 'ERROR_DIRECTORY_NOT_WRITABLE';
            }
        }

        // FILES
        if ($error == null) {
            $fileUploaded = false;

            if (count($_FILES)) {
                foreach ($_FILES as $file) {
                    if ($file['name'] == $filename) {
                        $uploaded_size = $file['size'];
                        $uploaded_type = $file['type'];

                        if ($file['error'] === UPLOAD_ERR_OK) {
                            try {
                                $tempPath = $this->_productMediaConfig->getTmpMediaPath(basename($file['tmp_name']));

                                $this->_mediaDirectory->create(dirname($tempPath));

                                $tempAbsolutePath = $this->_mediaDirectory->getAbsolutePath($tempPath);

                                $r = move_uploaded_file($file['tmp_name'], $tempAbsolutePath);
                                $this->_mediaDirectory->copyFile($tempPath, $filePath);

                                $fileUploaded = true;
                            } catch (\Exception $e) {
                                $error = $e->getMessage();
                            }
                        } else {
                            $error = 'ERROR_UPLOAD_ERR ' . $file['error'];
                        }
                    }
                }
            }

            if ($fileUploaded) {
                touch($absoluteFilePath, $lastModificationTime);
                $remoteLastModificationTime = filemtime($absoluteFilePath);
                if ($type == 'product') {
                    $imagesDeletedInCache = array();

                    $search = $base . DIRECTORY_SEPARATOR . 'cache' . DIRECTORY_SEPARATOR . '*' . DIRECTORY_SEPARATOR . '*' . DIRECTORY_SEPARATOR . '*' . DIRECTORY_SEPARATOR . '*' . $fileNameWithDispretionPath;  //media\catalog\product\cache\1\small_image\135x\9df78eab33525d08d6e5fb8d27136e95\y\u\yulips.jpg
                    $paths1 = glob($search);

                    $search = $base . DIRECTORY_SEPARATOR . 'cache' . DIRECTORY_SEPARATOR . '*' . DIRECTORY_SEPARATOR . '*' . DIRECTORY_SEPARATOR . '*' . $fileNameWithDispretionPath;         //media\catalog\product\cache\1\image\9df78eab33525d08d6e5fb8d27136e95\y\u\yulips.jpg
                    $paths2 = glob($search);

                    $imagesDeletedInCache = array_merge($paths1, $paths2);

                    foreach ($imagesDeletedInCache as $file) {
                        @unlink($file);
                    }

                    if ($this->_imageResize) {
                        $this->_imageResize->resizeFromImageName($fileNameWithDispretionPath);
                    }
                }
            }
        }

        $remoteFileSize = 0;

        if (file_exists($absoluteFilePath)) {
            $remoteFileSize = filesize($absoluteFilePath);
        }

        $fileNameWithDispretionPath = str_replace(DIRECTORY_SEPARATOR, '/', $fileNameWithDispretionPath);

        if ($error) {
            $jsonRpcResult->error = new \stdClass();
            $jsonRpcResult->error->errorcode = $error;
            $jsonRpcResult->error->remotefilename = $fileNameWithDispretionPath;
            $jsonRpcResult->error->remotefilepath = $filePath;
            $jsonRpcResult->error->remoteabsolutefilepath = $absoluteFilePath;
            $jsonRpcResult->error->remotefilesize = $remoteFileSize;
            $jsonRpcResult->error->remotelastmodificationtime = $remoteLastModificationTime;
            $jsonRpcResult->error->noduplicatedfilename = $noDuplicatedFileName;
            $jsonRpcResult->error->base = $base;
        } else {
            $jsonRpcResult->result = new \stdClass();
            $jsonRpcResult->result->nbbytes = $nbBytes;
            $jsonRpcResult->result->remotefilename = $fileNameWithDispretionPath;
            $jsonRpcResult->result->remotefilepath = $filePath;
            $jsonRpcResult->result->remoteabsolutefilepath = $absoluteFilePath;
            $jsonRpcResult->result->remotefilesize = $remoteFileSize;
            $jsonRpcResult->result->remotelastmodificationtime = $remoteLastModificationTime;
            $jsonRpcResult->result->imagesDeletedInCache = $imagesDeletedInCache;
            $jsonRpcResult->result->base = $base;
        }
    }

    public function deleteImage($jsonRpcResult, $type, $filename)
    {
        $error = null;

        $base = '';
        if ($type == 'product') {
            $base = $this->_productMediaConfig->getBaseMediaPath();
        }
        if ($type == 'category') {
            $base = $this->_storeManager->getStore()->getBaseMediaDir() . DIRECTORY_SEPARATOR . 'catalog' . DIRECTORY_SEPARATOR . 'category';
        }

        $fileName = Uploader::getCorrectFileName($filename);
        $dispretionPath = Uploader::getDispretionPath($fileName);
        $fileNameWithDispretionPath = $dispretionPath . '/' . $fileName;

        $filePath = $base . $fileNameWithDispretionPath;

        if (!file_exists($filePath)) {
            $error = 'ERROR_FILE_DOESNT_EXIST';
        } else {
            if (!unlink($filePath)) {
                $error = 'ERROR_FILE_CANT_BE_DELETED';
            }
        }

        if ($error) {
            $jsonRpcResult->error = new \stdClass();
            $jsonRpcResult->error->errorcode = $error;
            $jsonRpcResult->error->remotefilename = $fileName;
            $jsonRpcResult->error->remotefilepath = $filePath;
        } else {
            $jsonRpcResult->result = new \stdClass();
            $jsonRpcResult->result->remotefilename = $fileName;
            $jsonRpcResult->result->remotefilepath = $filePath;
        }
    }
}